@@ -18,6 +18,9 @@
    file system semantics.  */
 
 #include "cvs.h"
+#include "lstat.h"
+#include "save-cwd.h"
+#include "xsize.h"
 
 static int deep_remove_dir (const char *path);
 
@@ -30,17 +33,18 @@ copy_file (const char *from, const char *to)
     struct stat sb;
     struct utimbuf t;
     int fdin, fdout;
+    ssize_t rsize;
 
-    TRACE ( 1, "copy(%s,%s)", from, to );
+    TRACE (TRACE_FUNCTION, "copy(%s,%s)", from, to);
 
     if (noexec)
 	return;
 
     /* If the file to be copied is a link or a device, then just create
        the new link or device appropriately. */
-    if (islink (from))
+    if ((rsize = islink (from)) > 0)
     {
-	char *source = xreadlink (from);
+	char *source = Xreadlink (from, rsize);
 	symlink (source, to);
 	free (source);
 	return;
@@ -49,7 +53,7 @@ copy_file (const char *from, const char *to)
     if (isdevice (from))
     {
 #if defined(HAVE_MKNOD) && defined(HAVE_STRUCT_STAT_ST_RDEV)
-	if( CVS_STAT( from, &sb ) < 0 )
+	if (stat (from, &sb) < 0)
 	    error (1, errno, "cannot stat %s", from);
 	mknod (to, sb.st_mode, sb.st_rdev);
 #else
@@ -88,11 +92,6 @@ copy_file (const char *from, const char *to)
 		    error (1, errno, "cannot write file %s for copying", to);
 		}
 	    }
-
-#ifdef HAVE_FSYNC
-	    if (fsync (fdout)) 
-		error (1, errno, "cannot fsync file %s after copying", to);
-#endif
 	}
 
 	if (close (fdin) < 0) 
@@ -101,102 +100,116 @@ copy_file (const char *from, const char *to)
 	    error (1, errno, "cannot close %s", to);
     }
 
-    /* now, set the times for the copied file to match those of the original */
+    /* preserve last access & modification times */
     memset ((char *) &t, 0, sizeof (t));
     t.actime = sb.st_atime;
     t.modtime = sb.st_mtime;
     (void) utime (to, &t);
 }
 
+
+
 /* FIXME-krp: these functions would benefit from caching the char * &
    stat buf.  */
 
 /*
- * Returns non-zero if the argument file is a directory, or is a symbolic
+ * Returns true if the argument file is a directory, or is a symbolic
  * link which points to a directory.
  */
-int
+bool
 isdir (const char *file)
 {
     struct stat sb;
 
-    if( CVS_STAT( file, &sb ) < 0 )
-	return (0);
-    return (S_ISDIR (sb.st_mode));
+    if (stat (file, &sb) < 0)
+	return false;
+    return S_ISDIR (sb.st_mode);
 }
 
+
+
 /*
- * Returns non-zero if the argument file is a symbolic link.
+ * Returns 0 if the argument file is not a symbolic link.
+ * Returns size of the link if it is a symbolic link.
  */
-int
+ssize_t
 islink (const char *file)
 {
+    ssize_t retsize = 0;
 #ifdef S_ISLNK
     struct stat sb;
 
-    if (CVS_LSTAT (file, &sb) < 0)
-	return (0);
-    return (S_ISLNK (sb.st_mode));
-#else
-    return (0);
+    if ((lstat (file, &sb) >= 0) && S_ISLNK (sb.st_mode))
+	retsize = sb.st_size;
 #endif
+    return retsize;
 }
 
+
+
 /*
- * Returns non-zero if the argument file is a block or
+ * Returns true if the argument file is a block or
  * character special device.
  */
-int
+bool
 isdevice (const char *file)
 {
     struct stat sb;
 
-    if (CVS_LSTAT (file, &sb) < 0)
-	return (0);
+    if (lstat (file, &sb) < 0)
+	return false;
 #ifdef S_ISBLK
     if (S_ISBLK (sb.st_mode))
-	return 1;
+	return true;
 #endif
 #ifdef S_ISCHR
     if (S_ISCHR (sb.st_mode))
-	return 1;
+	return true;
 #endif
-    return 0;
+    return false;
 }
 
+
+
 /*
- * Returns non-zero if the argument file exists.
+ * Returns true if the argument file exists.
  */
-int
+bool
 isfile (const char *file)
 {
-    return isaccessible(file, F_OK);
+    return isaccessible (file, F_OK);
 }
 
+
+
 /*
  * Returns non-zero if the argument file is readable.
  */
-int
+bool
 isreadable (const char *file)
 {
-    return isaccessible(file, R_OK);
+    return isaccessible (file, R_OK);
 }
 
+
+
 /*
  * Returns non-zero if the argument file is writable.
  */
-int
+bool
 iswritable (const char *file)
 {
-    return isaccessible(file, W_OK);
+    return isaccessible (file, W_OK);
 }
 
+
+
 /*
- * Returns non-zero if the argument file is accessable according to
+ * Returns true if the argument file is accessable according to
  * mode.  If compiled with SETXID_SUPPORT also works if cvs has setxid
  * bits set.
  */
-int
+bool
 isaccessible (const char *file, const int mode)
 {
 #ifdef SETXID_SUPPORT
@@ -206,19 +219,19 @@ isaccessible (const char *file, const int mode)
     int omask = 0;
     int uid, mask;
     
-    if( CVS_STAT( file, &sb ) == -1 )
-	return 0;
+    if (stat (file, &sb)== -1)
+	return false;
     if (mode == F_OK)
-	return 1;
+	return true;
 
     uid = geteuid();
     if (uid == 0)		/* superuser */
     {
 	if (!(mode & X_OK) || (sb.st_mode & (S_IXUSR|S_IXGRP|S_IXOTH)))
-	    return 1;
+	    return true;
 
 	errno = EACCES;
-	return 0;
+	return false;
     }
 	
     if (mode & R_OK)
@@ -242,26 +255,15 @@ isaccessible (const char *file, const int mode)
 
     mask = sb.st_uid == uid ? umask : sb.st_gid == getegid() ? gmask : omask;
     if ((sb.st_mode & mask) == mask)
-	return 1;
+	return true;
     errno = EACCES;
-    return 0;
-#else
-    return access(file, mode) == 0;
-#endif
+    return false;
+#else /* !SETXID_SUPPORT */
+    return access (file, mode) == 0;
+#endif /* SETXID_SUPPORT */
 }
 
-/*
- * Open a file and die if it fails
- */
-FILE *
-open_file (const char *name, const char *mode)
-{
-    FILE *fp;
 
-    if ((fp = fopen (name, mode)) == NULL)
-	error (1, errno, "cannot open %s", name);
-    return (fp);
-}
 
 /*
  * Make a directory and die if it fails
@@ -271,7 +273,7 @@ make_directory (const char *name)
 {
     struct stat sb;
 
-    if( CVS_STAT( name, &sb ) == 0 && ( !S_ISDIR( sb.st_mode ) ) )
+    if (stat (name, &sb) == 0 && (!S_ISDIR (sb.st_mode)))
 	    error (0, 0, "%s already exists but is not a directory", name);
     if (!noexec && mkdir (name, 0777) < 0)
 	error (1, errno, "cannot make directory %s", name);
@@ -335,10 +337,12 @@ xchmod (const char *fname, int writable)
     struct stat sb;
     mode_t mode, oumask;
 
-    if (preserve_perms)
+#ifdef PRESERVE_PERMISSIONS_SUPPORT
+    if (config->preserve_perms)
 	return;
+#endif /* PRESERVE_PERMISSIONS_SUPPORT */
 
-    if( CVS_STAT( fname, &sb ) < 0 )
+    if (stat (fname, &sb) < 0)
     {
 	if (!noexec)
 	    error (0, errno, "cannot stat %s", fname);
@@ -358,7 +362,7 @@ xchmod (const char *fname, int writable)
 	mode = sb.st_mode & ~(S_IWRITE | S_IWGRP | S_IWOTH) & ~oumask;
     }
 
-    TRACE ( 1, "chmod(%s,%o)", fname, (unsigned int) mode );
+    TRACE (TRACE_FUNCTION, "chmod(%s,%o)", fname, (unsigned int) mode);
 
     if (noexec)
 	return;
@@ -373,7 +377,7 @@ xchmod (const char *fname, int writable)
 void
 rename_file (const char *from, const char *to)
 {
-    TRACE ( 1, "rename(%s,%s)", from, to );
+    TRACE (TRACE_FUNCTION, "rename(%s,%s)", from, to);
 
     if (noexec)
 	return;
@@ -388,7 +392,7 @@ rename_file (const char *from, const char *to)
 int
 unlink_file (const char *f)
 {
-    TRACE ( 1, "unlink_file(%s)", f );
+    TRACE (TRACE_FUNCTION, "unlink_file(%s)", f);
 
     if (noexec)
 	return (0);
@@ -408,12 +412,10 @@ unlink_file_dir (const char *f)
 {
     struct stat sb;
 
-#ifdef SERVER_SUPPORT
-	/* This is called by the server parent process in contexts where
-	   it is not OK to send output (e.g. after we sent "ok" to the
-	   client).  */
-	if (!server_active)
-#endif
+    /* This is called by the server parent process in contexts where
+       it is not OK to send output (e.g. after we sent "ok" to the
+       client).  */
+    if (!server_active)
 	TRACE (TRACE_FUNCTION, "unlink_file_dir(%s)", f);
 
     if (noexec)
@@ -427,7 +429,7 @@ unlink_file_dir (const char *f)
        call to stat() and the call to unlink(), we'll still corrupt
        the filesystem.  Where is the Unix Haters Handbook when you need
        it?  */
-    if( CVS_STAT( f, &sb ) < 0 )
+    if (stat (f, &sb) < 0)
     {
 	if (existence_error (errno))
 	{
@@ -477,8 +479,7 @@ deep_remove_dir (const char *path)
 			    strcmp (dp->d_name, "..") == 0)
 		    continue;
 
-		buf = xmalloc (strlen (path) + strlen (dp->d_name) + 5);
-		sprintf (buf, "%s/%s", path, dp->d_name);
+		buf = Xasprintf ("%s/%s", path, dp->d_name);
 
 		/* See comment in unlink_file_dir explanation of why we use
 		   isdir instead of just calling unlink and checking the
@@ -570,9 +571,9 @@ xcmp (const char *file1, const char *file2)
     int fd1, fd2;
     int ret;
 
-    if (CVS_LSTAT (file1, &sb1) < 0)
+    if (lstat (file1, &sb1) < 0)
 	error (1, errno, "cannot lstat %s", file1);
-    if (CVS_LSTAT (file2, &sb2) < 0)
+    if (lstat (file2, &sb2) < 0)
 	error (1, errno, "cannot lstat %s", file2);
 
     /* If FILE1 and FILE2 are not the same file type, they are unequal. */
@@ -585,8 +586,8 @@ xcmp (const char *file1, const char *file2)
     if (S_ISLNK (sb1.st_mode) && S_ISLNK (sb2.st_mode))
     {
 	int result;
-	buf1 = xreadlink (file1);
-	buf2 = xreadlink (file2);
+	buf1 = Xreadlink (file1, sb1.st_size);
+	buf2 = Xreadlink (file2, sb2.st_size);
 	result = (strcmp (buf1, buf2) == 0);
 	free (buf1);
 	free (buf2);
@@ -702,7 +703,8 @@ cvs_temp_name (void)
  *	whatever system function is called to generate the temporary file name.
  *	The value of filename is undefined on error.
  */
-FILE *cvs_temp_file (char **filename)
+FILE *
+cvs_temp_file (char **filename)
 {
     char *fn;
     FILE *fp;
@@ -715,14 +717,14 @@ FILE *cvs_temp_file (char **filename)
 
     assert (filename != NULL);
 
-    fn = xmalloc (strlen (Tmpdir) + 11);
-    sprintf (fn, "%s/%s", Tmpdir, "cvsXXXXXX" );
+    fn = Xasprintf ("%s/%s", get_cvs_tmp_dir (), "cvsXXXXXX");
     fd = mkstemp (fn);
 
     /* a NULL return will be interpreted by callers as an error and
      * errno should still be set
      */
-    if (fd == -1) fp = NULL;
+    if (fd == -1)
+	fp = NULL;
     else if ((fp = CVS_FDOPEN (fd, "w+")) == NULL)
     {
 	/* Attempt to close and unlink the file since mkstemp returned
@@ -736,7 +738,8 @@ FILE *cvs_temp_file (char **filename)
 	errno = save_errno;
     }
 
-    if (fp == NULL) free (fn);
+    if (fp == NULL)
+	free (fn);
 
     /* mkstemp is defined to open mode 0600 using glibc 2.0.7+.  There used
      * to be a complicated #ifdef checking the library versions here and then
@@ -758,92 +761,6 @@ FILE *cvs_temp_file (char **filename)
 
 
 
-#ifdef HAVE_READLINK
-/* char *
- * xreadlink ( const char *link )
- *
- * Like the X/OPEN and 4.4BSD readlink() function, but allocates and returns
- * its own buf.
- *
- * INPUTS
- *  link	The original path.
- *
- * RETURNS
- *  The resolution of the final symbolic link in the path.
- *
- * ERRORS
- *  This function exits with a fatal error if it fails to read the link for
- *  any reason.
- */
-char *
-xreadlink (const char *link)
-{
-    char *file = NULL;
-    int buflen = 128;
-    int link_name_len;
-
-    /* Get the name of the file to which `from' is linked.
-       FIXME: what portability issues arise here?  Are readlink &
-       ENAMETOOLONG defined on all systems? -twp */
-    do
-    {
-	file = xrealloc (file, buflen);
-	link_name_len = readlink (link, file, buflen - 1);
-	buflen *= 2;
-    }
-    while (link_name_len < 0 && errno == ENAMETOOLONG);
-
-    if (link_name_len < 0)
-	error (1, errno, "cannot readlink %s", link);
-
-    file[link_name_len] = '\0';
-
-    return file;
-}
-#endif /* HAVE_READLINK */
-
-
-
-/* char *
- * xresolvepath ( const char *path )
- *
- * Like xreadlink(), but resolve all links in a path.
- *
- * INPUTS
- *  path	The original path.
- *
- * RETURNS
- *  The path with any symbolic links expanded.
- *
- * ERRORS
- *  This function exits with a fatal error if it fails to read the link for
- *  any reason.
- */
-char *
-xresolvepath (const char *path)
-{
-    char *hardpath;
-    char *owd;
-
-    assert ( isdir ( path ) );
-
-    /* FIXME - If HAVE_READLINK is defined, we should probably walk the path
-     * bit by bit calling xreadlink().
-     */
-
-    owd = xgetwd();
-    if ( CVS_CHDIR ( path ) < 0)
-	error ( 1, errno, "cannot chdir to %s", path );
-    if ( ( hardpath = xgetwd() ) == NULL )
-	error (1, errno, "cannot getwd in %s", path);
-    if ( CVS_CHDIR ( owd ) < 0)
-	error ( 1, errno, "cannot chdir to %s", owd );
-    free (owd);
-    return hardpath;
-}
-
-
-
 /* Return a pointer into PATH's last component.  */
 const char *
 last_component (const char *path)
@@ -891,11 +808,7 @@ get_homedir (void)
     if (home != NULL)
 	return home;
 
-    if (
-#ifdef SERVER_SUPPORT
-	!server_active &&
-#endif
-	(env = getenv ("HOME")) != NULL)
+    if (!server_active && (env = getenv ("HOME")) != NULL)
 	home = env;
     else if ((pw = (struct passwd *) getpwuid (getuid ()))
 	     && pw->pw_dir)
@@ -919,8 +832,7 @@ get_homedir (void)
 char *
 strcat_filename_onto_homedir (const char *dir, const char *file)
 {
-    char *path = xmalloc (strlen (dir) + 1 + strlen(file) + 1);
-    sprintf (path, "%s/%s", dir, file);
+    char *path = Xasprintf ("%s/%s", dir, file);
     return path;
 }
 
@@ -937,35 +849,30 @@ expand_wild (int argc, char **argv, int *pargc, char ***pargv)
 	return;
     }
     *pargc = argc;
-    *pargv = xmalloc (xtimes (argc, sizeof (char *)));
+    *pargv = xnmalloc (argc, sizeof (char *));
     for (i = 0; i < argc; ++i)
 	(*pargv)[i] = xstrdup (argv[i]);
 }
 
 
 
-#ifdef SERVER_SUPPORT
-/* Case-insensitive string compare.  I know that some systems
-   have such a routine, but I'm not sure I see any reasons for
-   dealing with the hair of figuring out whether they do (I haven't
-   looked into whether this is a performance bottleneck; I would guess
-   not).  */
-int
-cvs_casecmp (const char *str1, const char *str2)
-{
-    const char *p;
-    const char *q;
-    int pqdiff;
+static char *tmpdir_env;
 
-    p = str1;
-    q = str2;
-    while ((pqdiff = tolower (*p) - tolower (*q)) == 0)
-    {
-	if (*p == '\0')
-	    return 0;
-	++p;
-	++q;
-    }
-    return pqdiff;
+/* Return path to temp directory.
+ */
+const char *
+get_system_temp_dir (void)
+{
+    if (!tmpdir_env) tmpdir_env = getenv (TMPDIR_ENV);
+    return tmpdir_env;
+}
+
+
+
+void
+push_env_temp_dir (void)
+{
+    const char *tmpdir = get_cvs_tmp_dir ();
+    if (tmpdir_env && strcmp (tmpdir_env, tmpdir))
+	setenv (TMPDIR_ENV, tmpdir, 1);
 }
-#endif /* SERVER_SUPPORT */
