@@ -1,6 +1,11 @@
 /*
- * Copyright (c) 1992, Brian Berliner and Jeff Polk
- * Copyright (c) 1989-1992, Brian Berliner
+ * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
+ *
+ * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
+ *                                  and others.
+ *
+ * Portions Copyright (C) 1992, Brian Berliner and Jeff Polk
+ * Portions Copyright (C) 1989-1992, Brian Berliner
  * 
  * You may distribute under the terms of the GNU General Public License as
  * specified in the README file that comes with the CVS source distribution.
@@ -13,6 +18,7 @@
  */
 
 #include "cvs.h"
+#include <assert.h>
 
 /* This structure holds information parsed from the -r option.  */
 
@@ -122,8 +128,8 @@ static int log_fileproc (void *callerdat, struct file_info *finfo);
 static struct option_revlist *log_parse_revlist (const char *);
 static void log_parse_date (struct log_data *, const char *);
 static void log_parse_list (List **, const char *);
-static struct revlist *log_expand_revlist (RCSNode *, struct option_revlist *,
-                                           int);
+static struct revlist *log_expand_revlist (RCSNode *, char *,
+                                           struct option_revlist *, int);
 static void log_free_revlist (struct revlist *);
 static int log_version_requested (struct log_data *, struct revlist *,
 					 RCSNode *, RCSVers *);
@@ -148,12 +154,14 @@ static const char *const log_usage[] =
     "Usage: %s %s [-lRhtNb] [-r[revisions]] [-d dates] [-s states]\n",
     "    [-w[logins]] [files...]\n",
     "\t-l\tLocal directory only, no recursion.\n",
-    "\t-R\tOnly print name of RCS file.\n",
+    "\t-b\tOnly list revisions on the default branch.\n",
     "\t-h\tOnly print header.\n",
+    "\t-R\tOnly print name of RCS file.\n",
     "\t-t\tOnly print header and descriptive text.\n",
     "\t-N\tDo not list tags.\n",
-    "\t-S\tDo not print name/header if no revisions selected.\n",
-    "\t-b\tOnly list revisions on the default branch.\n",
+    "\t-S\tDo not print name/header if no revisions selected.  -d, -r,\n",
+    "\t\t-s, & -w have little effect in conjunction with -b, -h, -R, and\n",
+    "\t\t-t without this option.\n",
     "\t-r[revisions]\tA comma-separated list of revisions to print:\n",
     "\t   rev1:rev2   Between rev1 and rev2, including rev1 and rev2.\n",
     "\t   rev1::rev2  Between rev1 and rev2, excluding rev1.\n",
@@ -505,8 +513,7 @@ rlog_proc (int argc, char **argv, char *xwhere, char *mwhere, char *mfile,
 	    }
 
 	    /* take care of the rest */
-	    path = xmalloc (strlen (repository) + strlen (mfile) + 5);
-	    (void)sprintf (path, "%s/%s", repository, mfile);
+	    path = Xasprintf ("%s/%s", repository, mfile);
 	    if (isdir (path))
 	    {
 		/* directory means repository gets the dir tacked on */
@@ -789,21 +796,30 @@ log_fileproc (void *callerdat, struct file_info *finfo)
 {
     struct log_data *log_data = callerdat;
     Node *p;
+    char *baserev;
     int selrev = -1;
     RCSNode *rcsfile;
     char buf[50];
     struct revlist *revlist = NULL;
     struct log_data_and_rcs log_data_and_rcs;
 
-    if ((rcsfile = finfo->rcs) == NULL)
+    rcsfile = finfo->rcs;
+    p = findnode (finfo->entries, finfo->file);
+    if (p != NULL)
+    {
+	Entnode *e = p->data;
+	baserev = e->version;
+	if (baserev[0] == '-') ++baserev;
+    }
+    else
+	baserev = NULL;
+
+    if (rcsfile == NULL)
     {
 	/* no rcs file.  What *do* we know about this file? */
-	p = findnode (finfo->entries, finfo->file);
-	if (p != NULL)
+	if (baserev != NULL)
 	{
-	    Entnode *e = p->data;
-
-	    if (e->version[0] == '0' && e->version[1] == '\0')
+	    if (baserev[0] == '0' && baserev[1] == '\0')
 	    {
 		if (!really_quiet)
 		    error (0, 0, "%s has been added, but not committed",
@@ -826,7 +842,7 @@ log_fileproc (void *callerdat, struct file_info *finfo)
 
 	/* Turn any symbolic revisions in the revision list into numeric
 	   revisions.  */
-	revlist = log_expand_revlist (rcsfile, log_data->revlist,
+	revlist = log_expand_revlist (rcsfile, baserev, log_data->revlist,
 				      log_data->default_branch);
 	if (log_data->sup_header
             || (!log_data->header && !log_data->long_header))
@@ -1016,8 +1032,8 @@ log_fileproc (void *callerdat, struct file_info *finfo)
  * Expand any symbolic revisions.
  */
 static struct revlist *
-log_expand_revlist (RCSNode *rcs, struct option_revlist *revlist,
-                    int default_branch)
+log_expand_revlist (RCSNode *rcs, char *baserev,
+                    struct option_revlist *revlist, int default_branch)
 {
     struct option_revlist *r;
     struct revlist *ret, **pr;
@@ -1036,8 +1052,19 @@ log_expand_revlist (RCSNode *rcs, struct option_revlist *revlist,
 	    /* If both first and last are NULL, it means that we want
 	       just the head of the default branch, which is RCS_head.  */
 	    nr->first = RCS_head (rcs);
-	    nr->last = xstrdup (nr->first);
-	    nr->fields = numdots (nr->first) + 1;
+	    if (!nr->first)
+	    {
+		if (!really_quiet)
+		    error (0, 0, "No head revision in archive `%s'.",
+		           rcs->path);
+		nr->last = NULL;
+		nr->fields = 0;
+	    }
+	    else
+	    {
+		nr->last = xstrdup (nr->first);
+		nr->fields = numdots (nr->first) + 1;
+	    }
 	}
 	else if (r->branchhead)
 	{
@@ -1057,10 +1084,11 @@ log_expand_revlist (RCSNode *rcs, struct option_revlist *revlist,
 		    free (branch);
 		}
 	    }
-	    if (nr->first == NULL && !really_quiet)
+	    if (!nr->first)
 	    {
-		error (0, 0, "warning: no branch `%s' in `%s'",
-		       r->first, rcs->print_path);
+		if (!really_quiet)
+		    error (0, 0, "warning: no branch `%s' in `%s'",
+			   r->first, rcs->print_path);
 		nr->last = NULL;
 		nr->fields = 0;
 	    }
@@ -1076,7 +1104,9 @@ log_expand_revlist (RCSNode *rcs, struct option_revlist *revlist,
 		nr->first = xstrdup (r->first);
 	    else
 	    {
-		if (RCS_nodeisbranch (rcs, r->first))
+		if (baserev && strcmp (r->first, TAG_BASE) == 0)
+		    nr->first = xstrdup (baserev);
+		else if (RCS_nodeisbranch (rcs, r->first))
 		    nr->first = RCS_whatbranch (rcs, r->first);
 		else
 		    nr->first = RCS_gettag (rcs, r->first, 1, NULL);
@@ -1094,7 +1124,9 @@ log_expand_revlist (RCSNode *rcs, struct option_revlist *revlist,
 		nr->last = xstrdup (r->last);
 	    else
 	    {
-		if (RCS_nodeisbranch (rcs, r->last))
+		if (baserev && strcmp (r->last, TAG_BASE) == 0)
+		    nr->last = xstrdup (baserev);
+		else if (RCS_nodeisbranch (rcs, r->last))
 		    nr->last = RCS_whatbranch (rcs, r->last);
 		else
 		    nr->last = RCS_gettag (rcs, r->last, 1, NULL);
@@ -1120,6 +1152,7 @@ log_expand_revlist (RCSNode *rcs, struct option_revlist *revlist,
 
 		    nr->first = xstrdup (nr->last);
 		    cp = strrchr (nr->first, '.');
+		    assert (cp);
 		    strcpy (cp + 1, "0");
 		}
 	    }
@@ -1134,6 +1167,7 @@ log_expand_revlist (RCSNode *rcs, struct option_revlist *revlist,
 		    char *cp;
 
 		    cp = strrchr (nr->last, '.');
+		    assert (cp);
 		    *cp = '\0';
 		}
 	    }
@@ -1175,9 +1209,7 @@ log_expand_revlist (RCSNode *rcs, struct option_revlist *revlist,
 		{
 		    if (nr->fields <= dots2 && (nr->fields & 1))
 		    {
-			char *p = xmalloc (strlen (nr->first) + 3);
-			strcpy (p, nr->first);
-			strcat (p, ".0");
+			char *p = Xasprintf ("%s.0", nr->first);
 			free (nr->first);
 			nr->first = p;
 			++nr->fields;
@@ -1233,7 +1265,9 @@ log_expand_revlist (RCSNode *rcs, struct option_revlist *revlist,
 	    char *cp;
 
 	    nr->first = xstrdup (rcs->head);
+	    assert (nr->first);
 	    cp = strrchr (nr->first, '.');
+	    assert (cp);
 	    *cp = '\0';
 	}
 	nr->last = xstrdup (nr->first);
@@ -1610,11 +1644,22 @@ log_version (struct log_data *log_data, struct revlist *revlist, RCSNode *rcs,
 
     if (padd != NULL)
     {
+	assert (pdel);
 	cvs_output_tagged ("text", "  lines: +");
 	cvs_output_tagged ("text", padd->data);
 	cvs_output_tagged ("text", " -");
 	cvs_output_tagged ("text", pdel->data);
+        cvs_output_tagged ("text", ";");
     }
+
+    p = findnode(ver->other_delta,"commitid");
+    if(p && p->data)
+    {
+        cvs_output_tagged ("text", "  commitid: ");
+	cvs_output_tagged ("text", p->data);
+	cvs_output_tagged ("text", ";");
+    }
+
     cvs_output_tagged ("newline", NULL);
 
     if (ver->branches != NULL)
